@@ -4,10 +4,11 @@ import operator
 import os
 import unittest
 import time
+from textwrap import dedent
 
 try:
     import numpy as np
-except ImportError:
+except ModuleNotFoundError:
     if os.getenv('PARAM_TEST_NUMPY','0') == '1':
         raise ImportError("PARAM_TEST_NUMPY=1 but numpy not available.")
     else:
@@ -15,7 +16,7 @@ except ImportError:
 
 try:
     import pandas as pd
-except ImportError:
+except ModuleNotFoundError:
     if os.getenv('PARAM_TEST_PANDAS','0') == '1':
         raise ImportError("PARAM_TEST_PANDAS=1 but pandas not available.")
     else:
@@ -24,8 +25,8 @@ except ImportError:
 import param
 import pytest
 
-from param.parameterized import Skip
-from param.reactive import bind, rx
+from param.parameterized import Skip, Parameter
+from param.reactive import bind, rx, reactive_ops
 
 from .utils import async_wait_until
 
@@ -107,8 +108,8 @@ def test_reactive_logic_binary_ops(op):
 
 @pytest.mark.parametrize('op', LOGIC_UNARY_OPERATORS)
 def test_reactive_logic_unary_ops(op):
-    assert op(rx(True)).rx.value == op(True)
-    assert op(rx(False)).rx.value == op(False)
+    assert op(rx(1)).rx.value == op(1)
+    assert op(rx(0)).rx.value == op(0)
 
 @pytest.mark.parametrize('op', LOGIC_BINARY_OPERATORS)
 def test_reactive_logic_binary_ops_reverse(op):
@@ -125,6 +126,26 @@ def test_reactive_getitem_list():
     assert rx([1, 2, 3])[1].rx.value == 2
     assert rx([1, 2, 3])[2].rx.value == 3
 
+def test_reactive_getitem_list_with_slice():
+    i = rx(1)
+    j = rx(5)
+    lst = list(range(10))
+    lstx = rx(lst)
+    sx = lstx[i: j]
+    assert sx.rx.value == lst[i.rx.value: j.rx.value]
+    i.rx.value = 2
+    assert sx.rx.value == lst[i.rx.value: j.rx.value]
+
+def test_reactive_getitem_numpy_with_tuple():
+    i = rx(0)
+    j = rx(1)
+    arr = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+    arrx = rx(arr)
+    selx = arrx[i, j]
+    assert selx.rx.value == arr[i.rx.value, j.rx.value]
+    i.rx.value = 1
+    assert selx.rx.value == arr[i.rx.value, j.rx.value]
+
 @pytest.mark.parametrize('ufunc', NUMPY_UFUNCS)
 def test_numpy_ufunc(ufunc):
     l = [1, 2, 3]
@@ -570,7 +591,7 @@ async def test_reactive_gen():
 async def test_reactive_gen_pipe():
     def gen(val):
         yield val+1
-        time.sleep(0.05)
+        time.sleep(0.1)
         yield val+2
 
     rxv = rx(0)
@@ -588,7 +609,7 @@ async def test_reactive_gen_pipe():
 async def test_reactive_gen_with_dep():
     def gen(i):
         yield i+1
-        time.sleep(0.05)
+        time.sleep(0.1)
         yield i+2
 
     irx = rx(0)
@@ -604,7 +625,7 @@ async def test_reactive_gen_with_dep():
 async def test_reactive_gen_pipe_with_dep():
     def gen(value, i):
         yield value+i+1
-        time.sleep(0.05)
+        time.sleep(0.1)
         yield value+i+2
 
     irx = rx(0)
@@ -709,3 +730,46 @@ def test_root_invalidation():
     arx.rx.value = 'd'
     assert expr.rx.value == 'DC'
     assert computed == ['1', '2', '2', '1']
+
+
+def test_ensure_ref_can_update_by_watcher_of_same_parameter():
+    # https://github.com/holoviz/param/pull/929
+
+    class W(param.Parameterized):
+        value = param.String()
+
+
+    class T(param.Parameterized):
+        lst = param.List(allow_refs=True, allow_None=True)
+
+        @param.depends("lst", watch=True)
+        def test(self):
+            lst = self.lst or range(5)
+            items = [W(value=str(i)) for i in lst]
+            with param.discard_events(self):
+                self.lst = param.rx(items).rx.resolve()
+            self.items = items
+
+    def transform(obj):
+        if isinstance(obj, W):
+            return obj.param.value
+        return obj
+
+
+    param.reactive.register_reference_transform(transform)
+
+    t = T()
+    t.lst = list("ABCDE")
+    t.items[1].value = "TEST"
+    assert t.lst[1] == "TEST"
+
+def test_reactive_callback_resolve_accessor():
+    df = pd.DataFrame({"name": ["Bill", "Bob"]})
+    dfx = rx(df)
+    out = dfx["name"].str._callback()
+    assert out is df["name"].str
+
+def test_docstrings_in_sync():
+    # The docstring needs to be explicitly written to work with LSP.
+    assert dedent(reactive_ops.__doc__) == dedent(Parameter.rx.__doc__)
+    assert dedent(reactive_ops.__doc__) == dedent(rx.rx.__doc__)
